from typing import Literal, List as typing_list, Dict as typing_dict, Optional
from .import_aron_slice import import_aron_slice


armarx_aron = import_aron_slice()
type_dto = armarx_aron.type.dto

AronIceTypeMaybe = Literal["NONE", "OPTIONAL", "RAW_PTR", "SHARED_PTR", "UNIQUE_PTR"]

AronQuaternionElementTyp = Literal["FLOAT64", "FLOAT32"]
AronQuaternionDefaultValue = Literal["ONES", "ZEROS", "DEFAULT"] | str

AronIceMatrixDefaultValue = Literal["IDENTITY", "ONES", "ZEROS", "DEFAULT"] | str
AronIceMatrixElementType = Literal[
    "UINT8", "UINT16", "UINT32", "INT8", "INT16", "INT32", "INT64", "FLOAT32", "FLOAT64"
]

AronIceNDArrayElementType = Literal[
    "UINT8", "INT8", "UINT16", "INT16", "UINT32", "INT32", "FLOAT32", "FLOAT64"
]
AronIceNDArrayDefaultValue = Literal["ONES", "ZEROS", "DEFAULT"] | str

AronIceEnumIntDefaultValue = Literal["DEFAULT"] | str

AronIcePointCloudVoxelType = Literal[
    "POINT_XYZ",
    "POINT_XYZI",
    "POINT_XYZL",
    "POINT_XYZRGB",
    "POINT_XYZRGBL",
    "POINT_XYZRGBA",
    "POINT_XYZHSV",
]
AronIcePointCloudDefaultValue = Literal["IDENTITY", "ONES", "ZEROS", "DEFAULT"] | str

AronIceImagePixelType = Literal["RGB24", "DEPTH32"]
AronIceImageDefaultValue = Literal["IDENTITY", "ONES", "ZEROS", "DEFAULT"] | str


class AronTypeIceTypes:
    _maybe = {
        "NONE": armarx_aron.type.Maybe.NONE,
        "OPTIONAL": armarx_aron.type.Maybe.OPTIONAL,
        "RAW_PTR": armarx_aron.type.Maybe.RAW_PTR,
        "SHARED_PTR": armarx_aron.type.Maybe.SHARED_PTR,
        "UNIQUE_PTR": armarx_aron.type.Maybe.UNIQUE_PTR,
    }
    _quaternion_element_type = {
        "FLOAT64": armarx_aron.type.quaternion.ElementType.FLOAT64,
        "FLOAT32": armarx_aron.type.quaternion.ElementType.FLOAT32,
    }
    _quaternion_default_value = {
        "ONES": armarx_aron.type.quaternion.default_value.ONES,
        "ZEROS": armarx_aron.type.quaternion.default_value.ZEROS,
        "DEFAULT": armarx_aron.type.quaternion.default_value.DEFAULT,
    }
    _matrix_element_type = {
        "UINT8": armarx_aron.type.matrix.ElementType.UINT8,
        "UINT16": armarx_aron.type.matrix.ElementType.UINT16,
        "UINT32": armarx_aron.type.matrix.ElementType.UINT32,
        "INT8": armarx_aron.type.matrix.ElementType.INT8,
        "INT16": armarx_aron.type.matrix.ElementType.INT16,
        "INT32": armarx_aron.type.matrix.ElementType.INT32,
        "INT64": armarx_aron.type.matrix.ElementType.INT64,
        "FLOAT32": armarx_aron.type.matrix.ElementType.FLOAT32,
        "FLOAT64": armarx_aron.type.matrix.ElementType.FLOAT64,
    }
    _matrix_default_value = {
        "IDENTITY": armarx_aron.type.matrix.default_value.IDENTITY,
        "ONES": armarx_aron.type.matrix.default_value.ONES,
        "ZEROS": armarx_aron.type.matrix.default_value.ZEROS,
        "DEFAULT": armarx_aron.type.matrix.default_value.DEFAULT,
    }
    _ndarray_element_type = {
        "UINT8": armarx_aron.type.ndarray.ElementType.UINT8,
        "INT8": armarx_aron.type.ndarray.ElementType.INT8,
        "UINT16": armarx_aron.type.ndarray.ElementType.UINT16,
        "INT16": armarx_aron.type.ndarray.ElementType.INT16,
        "UINT32": armarx_aron.type.ndarray.ElementType.UINT32,
        "INT32": armarx_aron.type.ndarray.ElementType.INT32,
        "FLOAT32": armarx_aron.type.ndarray.ElementType.FLOAT32,
        "FLOAT64": armarx_aron.type.ndarray.ElementType.FLOAT64,
    }
    _enum_int_default_value = {
        "DEFAULT": armarx_aron.type.aron_enum.default_value.DEFAULT
    }
    _point_cloud_voxel_type = {
        "POINT_XYZ": armarx_aron.type.pointcloud.VoxelType.POINT_XYZ,
        "POINT_XYZI": armarx_aron.type.pointcloud.VoxelType.POINT_XYZI,
        "POINT_XYZL": armarx_aron.type.pointcloud.VoxelType.POINT_XYZL,
        "POINT_XYZRGB": armarx_aron.type.pointcloud.VoxelType.POINT_XYZRGB,
        "POINT_XYZRGBL": armarx_aron.type.pointcloud.VoxelType.POINT_XYZRGBL,
        "POINT_XYZRGBA": armarx_aron.type.pointcloud.VoxelType.POINT_XYZRGBA,
        "POINT_XYZHSV": armarx_aron.type.pointcloud.VoxelType.POINT_XYZHSV,
    }
    _point_cloud_default_value = {
        "IDENTITY": armarx_aron.type.pointcloud.default_value.IDENTITY,
        "ONES": armarx_aron.type.pointcloud.default_value.ONES,
        "ZEROS": armarx_aron.type.pointcloud.default_value.ZEROS,
        "DEFAULT": armarx_aron.type.pointcloud.default_value.DEFAULT,
    }
    _image_pixel_type = {
        "RGB24": armarx_aron.type.image.PixelType.RGB24,
        "DEPTH32": armarx_aron.type.image.PixelType.DEPTH32,
    }
    _image_default_value = {
        "IDENTITY": armarx_aron.type.image.default_value.IDENTITY,
        "ONES": armarx_aron.type.image.default_value.ONES,
        "ZEROS": armarx_aron.type.image.default_value.ZEROS,
        "DEFAULT": armarx_aron.type.image.default_value.DEFAULT,
    }

    ARON_VERSION = armarx_aron.Version()

    Version = armarx_aron.Version

    Bool = type_dto.AronBool
    Int = type_dto.AronInt
    Long = type_dto.AronLong
    Float = type_dto.AronFloat
    Double = type_dto.AronDouble
    String = type_dto.AronString

    List = type_dto.List
    Tuple = type_dto.Tuple
    Pair = type_dto.Pair
    Quaternion = type_dto.Quaternion
    Dict = type_dto.Dict

    Object = type_dto.AronObject

    Matrix = type_dto.Matrix
    NDArray = type_dto.NDArray

    GenericType = type_dto.GenericType
    IntEnum = type_dto.IntEnum
    PointCloud = type_dto.PointCloud
    Image = type_dto.Image

    @classmethod
    def string(
        cls, default_value: Optional[str] = None, maybe: AronIceTypeMaybe = "NONE"
    ) -> String:
        if default_value is None:
            ret = cls.String(
                VERSION=cls.ARON_VERSION,
                maybe=cls._maybe[maybe],
            )
        else:
            ret = cls.String(
                VERSION=cls.ARON_VERSION,
                defaultValue=default_value,
                maybe=cls._maybe[maybe],
            )
        return ret

    @classmethod
    def bool(cls, default_value: Optional[bool] = None, maybe: AronIceTypeMaybe = "NONE") -> Bool:
        if default_value is None:
            ret = cls.Bool(
                VERSION=cls.ARON_VERSION,
                maybe=cls._maybe[maybe],
            )
        else:
            ret = cls.Bool(
                VERSION=cls.ARON_VERSION,
                defaultValue=default_value,
                maybe=cls._maybe[maybe],
            )
        return ret
    
    @classmethod
    def long(cls, default_value: Optional[int] = None, maybe: AronIceTypeMaybe = "NONE") -> Long:
        if default_value is None:
            ret = cls.Long(
                VERSION=cls.ARON_VERSION,
                defaultValue=default_value,
                maybe=cls._maybe[maybe],
            )
        else:
            ret = cls.Long(
                VERSION=cls.ARON_VERSION,
                maybe=cls._maybe[maybe],
            )
        return ret

    @classmethod
    def int(cls, default_value: Optional[int] = None, maybe: AronIceTypeMaybe = "NONE") -> Int:
        if default_value is None:
            ret = cls.Int(
                VERSION=cls.ARON_VERSION,
                maybe=cls._maybe[maybe],
            )
        else:
            ret = cls.Int(
                VERSION=cls.ARON_VERSION,
                defaultValue=default_value,
                maybe=cls._maybe[maybe],
            )
        return ret

    @classmethod
    def double(
        cls, default_value: Optional[float] = None, maybe: AronIceTypeMaybe = "NONE"
    ) -> Double:
        if default_value is None:
            ret = cls.Double(
                VERSION=cls.ARON_VERSION,
                maybe=cls._maybe[maybe],
            )
        else:
            ret = cls.Double(
                VERSION=cls.ARON_VERSION,
                defaultValue=default_value,
                maybe=cls._maybe[maybe],
            )
        return ret

    @classmethod
    def float(
        cls, default_value: Optional[float] = None, maybe: AronIceTypeMaybe = "NONE"
    ) -> Float:
        if default_value is None:
            ret = cls.Float(
                VERSION=cls.ARON_VERSION,
                maybe=cls._maybe[maybe],
            )
        else:
            ret = cls.Float(
                VERSION=cls.ARON_VERSION,
                defaultValue=default_value,
                maybe=cls._maybe[maybe],
            )
        return ret

    @classmethod
    def list(cls, accepted_type, maybe: AronIceTypeMaybe = "NONE") -> List:
        return cls.List(
            VERSION=cls.ARON_VERSION,
            maybe=cls._maybe[maybe],
            acceptedType=accepted_type,
        )

    @classmethod
    def tuple(cls, element_types, maybe: AronIceTypeMaybe = "NONE") -> Tuple:
        return cls.Tuple(
            VERSION=cls.ARON_VERSION,
            maybe=cls._maybe[maybe],
            elementTypes=element_types,
        )

    @classmethod
    def pair(
        cls, accepted_type1, accepted_type2, maybe: AronIceTypeMaybe = "NONE"
    ) -> Pair:
        return cls.Pair(
            VERSION=cls.ARON_VERSION,
            maybe=cls._maybe[maybe],
            acceptedType1=accepted_type1,
            acceptedType2=accepted_type2,
        )

    @classmethod
    def quaternion(
        cls,
        element_type: AronQuaternionElementTyp = "FLOAT64",
        maybe: AronIceTypeMaybe = "NONE",
        default_value: AronQuaternionDefaultValue = "DEFAULT",
    ) -> Quaternion:
        default = (
            cls._quaternion_default_value[default_value]
            if default_value in cls._quaternion_default_value
            else default_value
        )
        return cls.Quaternion(
            VERSION=cls.ARON_VERSION,
            maybe=cls._maybe[maybe],
            elementType=cls._quaternion_element_type[element_type],
            defaultValue=default,
        )

    @classmethod
    def dict(cls, accepted_type, maybe: AronIceTypeMaybe = "NONE") -> Dict:
        return cls.Dict(
            VERSION=cls.ARON_VERSION,
            maybe=cls._maybe[maybe],
            acceptedType=accepted_type,
        )

    @classmethod
    def matrix(
        cls,
        rows,
        cols,
        element_type: AronIceMatrixElementType = "FLOAT32",
        maybe: AronIceTypeMaybe = "NONE",
        default_value: AronIceMatrixDefaultValue = "DEFAULT",
    ) -> Matrix:
        default = (
            cls._matrix_default_value[default_value]
            if default_value in cls._matrix_default_value
            else default_value
        )
        element = (
            cls._matrix_element_type[element_type]
            if element_type in cls._matrix_element_type
            else element_type
        )
        return cls.Matrix(
            VERSION=cls.ARON_VERSION,
            elementType=element,
            maybe=cls._maybe[maybe],
            rows=rows,
            cols=cols,
            defaultValue=default,
        )

    @classmethod
    def nd_array(
        cls,
        ndimensions: int,
        element_type: AronIceNDArrayElementType = "FLOAT32",
        maybe: AronIceTypeMaybe = "NONE",
        default_value: AronIceNDArrayDefaultValue = "DEFAULT",
    ) -> NDArray:
        default = (
            cls._matrix_default_value[default_value]
            if default_value in cls._matrix_default_value
            else default_value
        )
        return cls.NDArray(
            VERSION=cls.ARON_VERSION,
            ndimensions=ndimensions,
            elementType=cls._ndarray_element_type[element_type],
            maybe=cls._maybe[maybe],
            defaultValue=default,
        )

    @classmethod
    def generic_type(cls, maybe: AronIceTypeMaybe = "NONE") -> GenericType:
        return cls.GenericType(VERSION=cls.ARON_VERSION, maybe=cls._maybe[maybe])

    @classmethod
    def int_enum(
        cls,
        name: str,
        accepted_values={},
        default_value: AronIceEnumIntDefaultValue = "DEFAULT",
        maybe: AronIceTypeMaybe = "NONE",
    ) -> IntEnum:
        default = (
            cls._enum_int_default_value[default_value]
            if default_value in cls._enum_int_default_value
            else default_value
        )
        return cls.IntEnum(
            VERSION=cls.ARON_VERSION,
            maybe=cls._maybe[maybe],
            acceptedValues=accepted_values,
            enumName=name,
            defaultValue=default,
        )

    @classmethod
    def point_cloud(
        cls,
        maybe: AronIceTypeMaybe = "NONE",
        voxel_type: AronIcePointCloudVoxelType = "POINT_XYZ",
        default_value: AronIcePointCloudDefaultValue = "DEFAULT",
    ) -> PointCloud:
        default = (
            cls._point_cloud_default_value[default_value]
            if default_value in cls._point_cloud_default_value
            else default_value
        )
        return cls.PointCloud(
            VERSION=cls.ARON_VERSION,
            maybe=cls._maybe[maybe],
            voxelType=cls._point_cloud_voxel_type[voxel_type],
            defaultValue=default,
        )

    @classmethod
    def image(
        cls,
        maybe: AronIceTypeMaybe = "NONE",
        pixel_type: AronIceImagePixelType = "RGB24",
        default_value: AronIceImageDefaultValue = "DEFAULT",
    ) -> Image:
        default = (
            cls._image_default_value[default_value]
            if default_value in cls._image_default_value
            else default_value
        )
        return cls.Image(
            VERSION=cls.ARON_VERSION,
            maybe=cls._maybe[maybe],
            pixelType=cls._image_pixel_type[pixel_type],
            defaultValue=default,
        )

    @classmethod
    def object(
        cls,
        object_name: str,
        templates: typing_list[str] = [],
        template_instantiations: typing_list[str] = [],
        element_types: typing_dict[str, GenericType] = {},
        maybe: AronIceTypeMaybe = "NONE",
        parent: Object = None,
    ) -> Object:
        return cls.Object(
            VERSION=cls.ARON_VERSION,
            maybe=cls._maybe[maybe],
            objectName=object_name,
            templates=templates,
            parent=parent,
            templateInstantiations=template_instantiations,
            elementTypes=element_types,
        )
