from dataclasses import dataclass, field

import numpy as np

# import python.aron.conv
import armarx_memory.aron.simox as simox
import armarx_memory.aron.simox.math
from armarx_memory.aron.data.variant import AronNDArray
from armarx_memory.aron.data.dto.aron_dto import AronDTOBase


@dataclass
class Pose(AronDTOBase):
    pose: np.ndarray = field(default_factory=lambda: np.diag(np.full(4,1, dtype=np.float32)))

    def x(self):
        return self.pose[0,3]
    
    def y(self):
        return self.pose[1,3]
    
    def z(self):
        return self.pose[2,3]
    
    def roll(self):
        rpy = simox.math.mat_to_rpy(self.pose)
        return rpy.r
    
    def pitch(self):
        rpy = simox.math.mat_to_rpy(self.pose)
        return rpy.p
    
    def yaw(self):
        rpy = simox.math.mat_to_rpy(self.pose)
        return rpy.y
    
    @classmethod
    def fromAron(cls, d: AronNDArray):
        dimensions = d.dimensions
        assert len(dimensions) == 3 and dimensions[0] == 4 and dimensions[1] == 4, "received wrong dimensions"

        pose = d.toNDArray()
        return cls(pose)
    
    def toNDArray(self) -> np.ndarray:
        n: np.ndarray = np.empty((0,1), dtype=np.float32)
        n = np.append(n, np.array([[self.roll]], dtype=np.float32))
        n = np.append(n, np.array([[self.pitch]], dtype=np.float32))
        n = np.append(n, np.array([[self.yaw]], dtype=np.float32))
        n = np.append(n, np.array([[self.x]], dtype=np.float32))
        n = np.append(n, np.array([[self.y]], dtype=np.float32))
        n = np.append(n, np.array([[self.z]], dtype=np.float32))
        return n