from dataclasses import dataclass

from armarx_memory.aron.data.variant import AronDict, AronInt, AronList, AronString
from armarx_memory.aron.data.dto.aron_dto import AronDTOBase


# TODO: Do we really need an aron type here?

@dataclass
class InstantiatedTypeBase(AronDTOBase):
    MY_TYPE_SLUG = "__aron_type__"    
    MY_COLS_SLUG = "__aron_cols__"
    
    type: str
    cols: int


@dataclass
class InstantiatedLeafType(InstantiatedTypeBase):
    MY_ADDITIONAL_SLUG = "__aron_additional_info__"

    additional_info: str = ""

    @classmethod
    def fromAron(cls, d: AronDict):
        t = d.at[cls.MY_TYPE_SLUG]
        assert isinstance(t, AronString)

        c = d.at[cls.MY_COLS_SLUG]
        assert isinstance(c, AronInt)

        a = d.at[cls.MY_ADDITIONAL_SLUG]
        assert isinstance(a, AronString)

        return cls(t.value, c.value, a.value)

    def toAron(self, path: list[str]):
        d: AronDict = AronDict(path=path)
        d.add(self.MY_TYPE_SLUG, AronString(path=path+[self.MY_TYPE_SLUG], value=self.type))
        d.add(self.MY_COLS_SLUG, AronInt(path=path+[self.MY_COLS_SLUG], value=self.cols))
        d.add(self.MY_ADDITIONAL_SLUG, AronString(path=path+[self.MY_ADDITIONAL_SLUG], value=self.additional_info))
        return d


@dataclass
class InstantiatedContainerType(InstantiatedTypeBase):
    MY_ELEMENTS_SLUG = "__aron_elements__"

    elements: dict | list # dict[str, InstantiatedLeafType | InstantiatedContainerType] | list[InstantiatedLeafType | InstantiatedContainerType]

    def at(self, path: list[str]):
        if len(path) == 0:
            return self
        elif len(path) > 1:
            if isinstance(self.elements, dict):
                return self.elements[path[0]].at(path[1:])
            elif isinstance(self.elements, list):
                return self.elements[int(path[0])].at(path[1:])
            else:
                raise ValueError(" Elements type not valid!")
        else:
            if isinstance(self.elements, dict):
                return self.elements[path[0]]
            elif isinstance(self.elements, list):
                return self.elements[int(path[0])]
            else:
                raise ValueError(" Elements type not valid!")

    @classmethod
    def fromAron(cls, d: AronDict):

        t = d.at[cls.MY_TYPE_SLUG]
        assert isinstance(t, AronString)

        e = d.at[cls.MY_ELEMENTS_SLUG]
        if (t.value == AronDict.IDENTIFICATOR):
            assert isinstance(e, AronDict)

            elements: dict = {}
            for k, v in e.elements.items():
                assert isinstance(v, AronDict)

                vt = v.at[cls.MY_TYPE_SLUG]
                assert isinstance(vt, AronString)

                if vt.value == AronDict.IDENTIFICATOR or vt.value == AronList.IDENTIFICATOR:
                    # again a container type
                    element = cls(v)
                    elements[k] = element
                else:
                    # a leaf type
                    element = InstantiatedLeafType.fromAron(v)
                    elements[k] = element
            return cls(t.value, elements)
        
        elif (t.value == AronList.IDENTIFICATOR):
            assert isinstance(e, AronList)

            elements = []
            for v in e.elements:
                assert isinstance(v, AronDict)

                vt = v.at[cls.MY_TYPE_SLUG]
                assert isinstance(vt, AronString)

                if vt.value == AronDict.IDENTIFICATOR or vt.value == AronList.IDENTIFICATOR:
                    # again a container type
                    element = cls(v)
                    elements += [element]
                else:
                    # a leaf type
                    element = InstantiatedLeafType.fromAron(v)
                    elements += [element]
            return cls(t.value, elements)
        else:
            raise ValueError("Type " + t.value + " not valid!")

    def toAron(self, path) -> AronDict:
        d: AronDict = AronDict(path=path)
        d.add(self.MY_TYPE_SLUG, AronString(path=path+[self.MY_TYPE_SLUG], value=self.type))

        if isinstance(self.elements, dict):
            _d: AronDict = AronDict(path=path+[self.MY_ELEMENTS_SLUG])
            for k, v in self.elements.items():
                _d.add(k, v.toAron(path=path+[self.MY_ELEMENTS_SLUG, k]))
            d.add(self.MY_ELEMENTS_SLUG, _d)
        elif isinstance(self.elements, list):
            _l: AronList = AronList(path=path+[self.MY_ELEMENTS_SLUG])
            for i, v in enumerate(self.elements):
                _l.add(i, v.toAron(path=path+[self.MY_ELEMENTS_SLUG, str(i)]))
            d.add(self.MY_ELEMENTS_SLUG, _l)
        else:
            raise ValueError("element type not supported")
        return d