from dataclasses import dataclass

from ...variant import AronDict
from armarx_memory.aron.data.variant import AronDict, AronDouble, AronLong
from armarx_memory.aron.data.dto.aron_dto import AronDTOBase


@dataclass
class Metadata(AronDTOBase):
    timeArrived: int = 0
    timeSent: int  = 0
    timeReferenced: int = 0
    timeStored: int = 0

    def setAllTo(self, ts: int) -> None:
        self.timeArrived = ts
        self.timeSent = ts
        self.timeReferenced = ts
        self.timeStored = ts

    @classmethod
    def fromAron(cls, d: AronDict):

        confidence = d.at("__ENTITY_METADATA__CONFIDENCE")
        assert isinstance(confidence, AronDouble)

        arrived = d.at("__ENTITY_METADATA__TIME_ARRIVED")
        assert isinstance(arrived, AronLong)

        referenced = d.at("__ENTITY_METADATA__TIME_REFERENCED")
        assert isinstance(referenced, AronLong)

        sent = d.at("__ENTITY_METADATA__TIME_SENT")
        assert isinstance(sent, AronLong)

        stored = d.at("__WRITER_METADATA__TIME_STORED")
        assert isinstance(stored, AronLong)

        return cls(timeArrived=arrived.value, timeSent=sent.value, timeReferenced=referenced.value, timeStored=stored.value)
    
    def toAron(self) -> AronDict:
        elements = {
            "__ENTITY_METADATA__CONFIDENCE" : AronDouble(1.0, "__ENTITY_METADATA__CONFIDENCE"),
            "__ENTITY_METADATA__TIME_ARRIVED": AronLong(self.timeArrived, "__ENTITY_METADATA__TIME_ARRIVED"),
            "__ENTITY_METADATA__TIME_REFERENCED": AronLong(self.timeReferenced, "__ENTITY_METADATA__TIME_REFERENCED"),
            "__ENTITY_METADATA__TIME_SENT": AronLong(self.timeSent, "__ENTITY_METADATA__TIME_SENT"),
            "__WRITER_METADATA__TIME_STORED": AronLong(self.timeStored, "__WRITER_METADATA__TIME_STORED")
        }
        aron_dict = AronDict(elements=elements, path=[])
        return aron_dict
    
    def toJSON(self):
        return self.toAron().toJSON()
