from dataclasses import dataclass, field

import numpy as np

from armarx_memory.aron.data.variant import AronDict, AronNDArray
from armarx_memory.aron.data.dto.aron_dto import AronDTOBase
from armarx_memory.aron.data.dto.common.instantiated_type import InstantiatedLeafType, InstantiatedContainerType


@dataclass
class ImageRGB(AronDTOBase):
    image: np.ndarray = field(default_factory=lambda: np.empty((0,1), dtype=np.float32))

    @classmethod
    def fromAron(cls, d: AronDict):

        img = d.at("image")
        assert isinstance(img, AronNDArray)

        return cls(img.toNDArray())
    
    def toAron(self) -> AronDict:
        root: AronDict = AronDict(path=[])
        image: AronNDArray = AronNDArray.fromNDArray(array=self.image, path=["image"])
        root.add("image", image)
        return root

    def toNDArray(self) -> tuple[np.ndarray, InstantiatedContainerType]:        
        image_structure = InstantiatedLeafType(type=AronNDArray.IDENTIFICATOR, cols=self.image.shape[0], additional_info="float32")
        structure = InstantiatedContainerType(type=AronDict.IDENTIFICATOR, cols=image_structure.cols, elements={"image": image_structure})

        return self.image, structure