import math

from armarx_memory.aron.data.variant import AronBool, AronInt, AronDouble, AronFloat, AronLong, AronNDArray, AronString
from armarx_memory.aron.data.visitors.visitor import RecursiveVisitorInterface


class ShapeCalculator(RecursiveVisitorInterface):

    def __init__(self) -> None:
        self.shape = None

    @staticmethod
    def _sanitize_shape(shape: tuple[int, ...]) -> tuple[int, ...]:
            # the amount of rows is unspecified because it depends on the number of members and cannot be known in advance
            ret: tuple[int, ...] = shape 
            if shape[-1] != 1:
                ret += (1,) # We enforce everything to have shape (0, ..., 1)
            return ret
    
    def _add_to_shape(self, shape: tuple[int, ...]) -> None:
        assert shape[-1] == 1, "The shape is wrong structured"

        if self.shape is None:
            self.shape = shape
            return
        
        assert shape[-1] == 1, "The self.shape is wrong structured"
        if self.shape[1:] == shape[1:]:
            self.shape = (self.shape[0] + shape[0], 1)
            return
        
        # otherwise we fall back to (n,1)
        if len(self.shape) != 2:
            result = math.prod([item for item in self.shape])
            self.shape = (result, 1)
        
        if len(shape) != 2:
            result = math.prod([item for item in shape])
            shape = (result, 1)
        self._add_to_shape(shape) # recursively add to shape


    def visitBool(self, d: AronBool) -> None:
        s = self._sanitize_shape((1,))
        self._add_to_shape(s)
            

    def visitInt(self, d: AronInt) -> None:
        s = self._sanitize_shape((1,))
        self._add_to_shape(s)

    def visitLong(self, d: AronLong) -> None:
        s = self._sanitize_shape((1,))
        self._add_to_shape(s)

    def visitFloat(self, d: AronFloat) -> None:
        s = self._sanitize_shape((1,))
        self._add_to_shape(s)

    def visitDouble(self, d: AronDouble) -> None:
        s = self._sanitize_shape((1,))
        self._add_to_shape(s)

    def visitString(self, d: AronString) -> None:
        pass # we ignore strings

    def visitNDArray(self, d: AronNDArray) -> None:
        s = self._sanitize_shape(tuple(d.dimensions[:-1])) # we remove the bytesize
        self._add_to_shape(s)