from armarx_memory.aron.data.variant import (AronData, AronBool, AronDouble, AronDict, AronInt, AronFloat, AronList, AronLong,
                                  AronNDArray, AronString)

class VisitorInterface:

    def visitBool(self, d: AronBool) -> None:
        pass
    
    def visitInt(self, d: AronInt) -> None:
        pass
    
    def visitLong(self, d: AronLong) -> None:
        pass
    
    def visitFloat(self, d: AronFloat) -> None:
        pass
    
    def visitDouble(self, d: AronDouble) -> None:
        pass
    
    def visitString(self, d: AronString) -> None:
        pass
    
    def visitNDArray(self, d: AronNDArray) -> None:
        pass
    
    def visitList(self, d: AronList) -> None:
        pass
    
    def visitDict(self, d: AronDict) -> None:
        pass
    
    def visit(self, d: AronData) -> None:
        if isinstance(d, AronDict):
            self.visitDict(d)
        if isinstance(d, AronList):
            self.visitList(d)
        if isinstance(d, AronNDArray):
            self.visitNDArray(d)
        if isinstance(d, AronInt):
            self.visitInt(d)
        if isinstance(d, AronLong):
            self.visitLong(d)
        if isinstance(d, AronFloat):
            self.visitLong(d)
        if isinstance(d, AronBool):
            self.visitBool(d)
        if isinstance(d, AronString):
            self.visitString(d)
        else:
            raise ValueError("No supported type")
    
    
class RecursiveVisitorInterface:

    def visitBool(self, d: AronBool) -> None:
        pass
    
    def visitInt(self, d: AronInt) -> None:
        pass
    
    def visitLong(self, d: AronLong) -> None:
        pass
    
    def visitFloat(self, d: AronFloat) -> None:
        pass
    
    def visitDouble(self, d: AronDouble) -> None:
        pass
    
    def visitString(self, d: AronString) -> None:
        pass
    
    def visitNDArray(self, d: AronNDArray) -> None:
        pass
    
    def visitListStart(self, d: AronList) -> None:
        pass
    
    def visitListEnd(self, d: AronList) -> None:
        pass
    
    def visitDictStart(self, d: AronDict) -> None:
        pass
    
    def visitDictEnd(self, d: AronDict) -> None:
        pass
    
    def visit(self, d: AronData):
        if isinstance(d, AronDict):
            self.visitDictStart(d)
            for el in d.elements.values():
                self.visit(el)
            self.visitDictEnd(d)
        elif isinstance(d, AronList):
            self.visitListStart(d)
            for el in d.elements:
                self.visit(el)
            self.visitListEnd(d)
        elif isinstance(d, AronNDArray):
            self.visitNDArray(d)
        elif isinstance(d, AronInt):
            self.visitInt(d)
        elif isinstance(d, AronLong):
            self.visitLong(d)
        elif isinstance(d, AronFloat):
            self.visitFloat(d)
        elif isinstance(d, AronBool):
            self.visitBool(d)
        elif isinstance(d, AronString):
            self.visitString(d)
        else:
            raise ValueError("No supported type. Got: " + str(d.__class__))