import os
from typing import TypeVar, Generic

from armarx_memory.ltm.base.provider_segment import ProviderSegment
from armarx_memory.ltm.util.filesystem_handler import escapeName, unescapeName
from armarx_memory.core.MemoryID import MemoryID


ProviderSegmentType = TypeVar('ProviderSegmentType')

class CoreSegment(Generic[ProviderSegmentType]):

    def __init__(self, id: MemoryID, basepath: str):
        self.id: MemoryID = id.getCoreSegmentID()
        self.basepath: str = basepath
        self.rootdir = self.basepath + "/" + escapeName(self.id.core_segment_name)
        self.providerSegments: dict[str, ProviderSegmentType] = {}

    def loadReferences(self):
        subdirs = [f.name for f in os.scandir(self.rootdir) if f.is_dir()]

        for subdirName in subdirs:

            # only load what's necessary
            if self.id.hasProviderSegmentName() and self.id.provider_segment_name != unescapeName(subdirName):
                continue

            self.addProviderSegment(self.id.changeProviderSegmentName(subdirName))
            self.providerSegments[subdirName].loadReferences()

    def addProviderSegment(self, id: MemoryID = None, providerSegment: ProviderSegment = None):
        if id != None and providerSegment != None:
            assert id == providerSegment.id, "You are trying to add a provider segment to the memory but the id given and the id of the provider segment object do not match"
        if providerSegment != None:
            self.providerSegments[providerSegment.id.provider_segment_name] = providerSegment
        elif id != None:
            child = ProviderSegment(id, self.rootdir)
            self.providerSegments[id.provider_segment_name] = child
        else:
            assert False, "You are trying to add a Provider Segment to this Core Segment, but you have neither provided an id nor a provider segment object"

        

    def for_each_provider_segment(self, fn):
        for _, providerSegment in self.providerSegments.items():
            fn(providerSegment)

    def for_each_entity(self, fn):
        for _, providerSegment in self.providerSegments.items():
            providerSegment.for_each_entity(fn)

    def for_each_snapshot(self, fn):
        for _, providerSegment in self.providerSegments.items():
            providerSegment.for_each_snapshot(fn)

    def for_each_instance(self, fn):
        for _, providerSegment in self.providerSegments.items():
            providerSegment.for_each_instance(fn)

    def for_each_data(self, fn):
        for _, providerSegment in self.providerSegments.items():
            providerSegment.for_each_data(fn)

    def len(self):
        length = 0
        for providerSegmentName, providerSegment in self.providerSegments.items():
            length += providerSegment.len()
        return length
