import os
from typing import Generic, TypeVar

from armarx_memory.core.MemoryID import MemoryID
from armarx_memory.ltm.base.core_segment import CoreSegment

from armarx_memory.ltm.util.filesystem_handler import escapeName, unescapeName

CoreSegmentType = TypeVar("CoreSegmentType")


class Memory(Generic[CoreSegmentType]):

    def __init__(self, id: MemoryID, basepath: str, exportName: str):
        self.id: MemoryID = id.getMemoryID()
        self.basepath: str = basepath
        self.exportName: str = exportName
        self.rootdir = self.basepath + "/" + escapeName(self.exportName) + "/" + escapeName(self.id.memory_name)
        self.coreSegments: dict[str, CoreSegmentType] = {}

    def loadReferences(self):
        subdirs = [f.name for f in os.scandir(self.rootdir) if f.is_dir()]

        for subdirName in subdirs:
            # only load what's necessary
            if self.id.hasCoreSegmentName() and self.id.core_segment_name != unescapeName(subdirName):
                continue

            self.addCoreSegment(self.id.changeCoreSegmentName(subdirName))
            self.coreSegments[subdirName].loadReferences()

    def addCoreSegment(self, id: MemoryID = None, coreSegment: CoreSegment = None):
        if id != None and coreSegment != None:
            assert id == coreSegment.id, "You are trying to add a core segment to the memory but the id given and the id of the core segment object do not match"
        if coreSegment != None:
            self.coreSegments[coreSegment.id.core_segment_name] = coreSegment
        elif id != None:
            child = CoreSegment(id, self.rootdir)
            self.coreSegments[id.core_segment_name] = child
        else:
            assert False, "You are trying to add a Core Segment to this memory, but you have neither provided an id nor a core segment object"
        

    def for_each_core_segment(self, fn):
        for _, coreSegment in self.coreSegments.items():
            fn(coreSegment)

    def for_each_provider_segment(self, fn):
        for _, coreSegment in self.coreSegments.items():
            coreSegment.for_each_provider_segment(fn)

    def for_each_entity(self, fn):
        for _, coreSegment in self.coreSegments.items():
            coreSegment.for_each_entity(fn)

    def for_each_snapshot(self, fn):
        for _, coreSegment in self.coreSegments.items():
            coreSegment.for_each_snapshot(fn)

    def for_each_instance(self, fn):
        for _, coreSegment in self.coreSegments.items():
            coreSegment.for_each_instance(fn)

    def for_each_data(self, fn):
        for _, coreSegment in self.coreSegments.items():
            coreSegment.for_each_data(fn)

    def len(self):
        length = 0
        for _, coreSegment in self.coreSegments.items():
            length += coreSegment.len()
        return length
