import os
from typing import TypeVar, Generic

from armarx_memory.core.MemoryID import MemoryID
from armarx_memory.ltm.base.entity import Entity
from armarx_memory.ltm.util.filesystem_handler import escapeName, unescapeName

EntityType = TypeVar('EntityType')

class ProviderSegment(Generic[EntityType]):

    def __init__(self, id: MemoryID, basepath: str):
        self.id: MemoryID = id.getProviderSegmentID()
        self.basepath: str = basepath
        self.rootdir = self.basepath + "/" + escapeName(self.id.provider_segment_name)
        self.entities: dict[str, EntityType] = {}

    def loadReferences(self):
        subdirs = [f.name for f in os.scandir(self.rootdir) if f.is_dir()]

        for subdirName in subdirs:

            # only load what's necessary
            if self.id.hasEntityName() and self.id.entity_name != unescapeName(subdirName):
                continue

            self.addEntity(self.id.changeEntityName(subdirName))
            self.entities[subdirName].loadReferences()

    def addEntity(self, id: MemoryID = None, entity: Entity = None):
        if id != None and entity != None:
            assert id == entity.id, "You are trying to add an entity to the provider segment but the id given and the id of the entity object do not match"
        if entity != None:
            self.entities[entity.id.entity_name] = entity
        elif id != None:
            child = Entity(id, self.rootdir)
            self.entities[id.entity_name] = child
        else:
            assert False, "You are trying to add an entity to this memory, but you have neither provided an id nor an entity object"



    def for_each_entity(self, fn):
        for _, entity in self.entities.items():
            fn(entity)

    def for_each_snapshot(self, fn):
        for _, entity in self.entities.items():
            entity.for_each_snapshot(fn)

    def for_each_instance(self, fn):
        for _, entity in self.entities.items():
            entity.for_each_instance(fn)

    def for_each_data(self, fn):
        for _, entity in self.entities.items():
            entity.for_each_data(fn)

    def len(self):
        return len(self.entities)

    def snapshotSize(self):
        length = 0
        for _, entity in self.entities.items():
            length += entity.size()
        return length