import os
import time

from armarx_memory.ltm.base.entity_snapshot import EntitySnapshot
from armarx_memory.ltm.base.memory import Memory
from armarx_memory.core.MemoryID import MemoryID
from armarx_memory.ltm.base.core_segment import CoreSegment
from armarx_memory.ltm.base.provider_segment import ProviderSegment
from armarx_memory.ltm.base.entity import Entity
from armarx_memory.ltm.base.entity_instance import EntityInstance

from armarx_memory.ltm.util.filesystem_handler import escapeName


class MemoryServer:

    def __init__(self, base_path: str, export_name: str):
        self.basepath: str = base_path
        self.exportName: str = export_name
        self.memories: dict[str, Memory] = {}
        self.load_memories()

    def load_memories(self):
        rootdir = self.basepath + "/" + escapeName(self.exportName)  # filepath to exported LTM
        subdirs = [f.name for f in os.scandir(rootdir) if f.is_dir()]  # names of memories in exported LTM

        for subdirName in subdirs:
            if subdirName.startswith("_"):
                print("Ignoring '" + subdirName + "'...")
                continue

            print("Create memory '" + subdirName + "'...")

            mid = MemoryID(subdirName)
            m = Memory(mid, self.basepath, self.exportName)

            self.memories[mid.memory_name] = m

    def loadReferences(self):
        for k, m in self.memories.items():
            print("Load references of memory '" + k + "'...")
            m.loadReferences()

    def get_entity(self, mid: MemoryID):
        assert mid.memory_name in self.memories.keys(), "Memory not found"
        m: Memory = self.memories[mid.memory_name]

        assert mid.core_segment_name in m.coreSegments.keys(), "Core segment not found"
        c: CoreSegment = m.coreSegments[mid.core_segment_name]

        assert mid.provider_segment_name in c.providerSegments.keys(), ("Provider segment " + mid.provider_segment_name +
                                                                      " not found. Got: " +
                                                                      str(c.providerSegments.keys()))
        p: ProviderSegment = c.providerSegments[mid.provider_segment_name]

        assert mid.entity_name in p.entities.keys(), "Entity not found"
        e: Entity = p.entities[mid.entity_name]

        return e


if __name__ == "__main__":
    pass



